!
!        Copyright (C) 2000-2016 the YAMBO team
!              http://www.yambo-code.org
!
! Authors (see AUTHORS file for details): AM
! 
! This file is distributed under the terms of the GNU 
! General Public License. You can redistribute it and/or 
! modify it under the terms of the GNU General Public 
! License as published by the Free Software Foundation; 
! either version 2, or (at your option) any later version.
!
! This program is distributed in the hope that it will 
! be useful, but WITHOUT ANY WARRANTY; without even the 
! implied warranty of MERCHANTABILITY or FITNESS FOR A 
! PARTICULAR PURPOSE.  See the GNU General Public License 
! for more details.
!
! You should have received a copy of the GNU General Public 
! License along with this program; if not, write to the Free 
! Software Foundation, Inc., 59 Temple Place - Suite 330,Boston, 
! MA 02111-1307, USA or visit http://www.gnu.org/copyleft/gpl.txt.
!
subroutine ypp_INIT_load(defs)
 !
 use it_m,          ONLY:it,initdefs,E_unit,G_unit,V_general,initmode,&
&                        Time_unit,Bfield_unit,V_nl_optics,T_unit
 use wave_func,     ONLY:wf_ng
 use D_lattice,     ONLY:Tel,Bose_Temp
 use electrons,     ONLY:n_spinor
 use interpolate,   ONLY:lpfac
 use YPP_non_linear, ONLY:NL_response,X_order,N_richardson 
 use YPP
#if defined _YPP_ELPH
 use YPP_ELPH
#endif
 !
 implicit none
 !
 type(initdefs)::defs
 integer       ::i_region
 !
 ! RunLevels
 !
 call it('r',defs,'bzgrids',   '[R] BZ Grid') 
 call it('r',defs,'K_grid',    '[R] K-grid analysis') 
 call it('r',defs,'Q_grid',    '[R] Q-grid analysis') 
 call it('r',defs,'dos',       '[R] DOS')
 !
 !
 call it('r',defs,'Random_Grid','[R] Random grids')
 call it('r',defs,'High_Symm',  '[R] High_Symmetry points and paths') 
 !
 call it('r',defs,'excitons',  '[R] Excitons')
 call it('r',defs,'electrons', '[R] Electrons (and holes)')
 call it('r',defs,'freehole',  '[R] Free hole position')
 call it('r',defs,'amplitude', '[R] Amplitude')
 call it('r',defs,'plot',         '[R] Plot')
 call it('r',defs,'density',      '[R] Density')
 call it('r',defs,'current',      '[R] Current')
 call it('r',defs,'wannier',      '[R] Wannier 90 I/O')
 call it('r',defs,'bnds',        '[R] Bands')
 call it('r',defs,'wavefunction', '[R] Wavefunction')
#if defined _YPP_ELPH
 call it('r',defs,'eliashberg',   '[R] Eliashberg')
 call it('r',defs,'phonons',      '[R] Phonons')
 call it('r',defs,'gkkp',         '[R] Electron-Phonon databases')
#endif
 call INIT_QP_ctl_load(defs,3)
 call it('r',defs,'spin',         '[R] Spin')
 call it('r',defs,'magnetization','[R] Magnetization')
 call it('r',defs,'kpts_map',     '[R] BZ map fine grid to coarse')
 call it('r',defs,'WFs_map',      '[R] WF map WFs with SOC to present WFs')
 call it('r',defs,'QPDBs', '[R] Quasi-particle databases')
 call it('r',defs,'QPDB_edit', '[R] Generation/editing')
 call it('r',defs,'QPDB_merge','[R] Mergering')
 call it('r',defs,'bxsf'   ,   '[R] Read BXSF files generated by Wannier90')    
 !
 call it('r',defs,'fixsyms',  '[R] Reduce Symmetries')
 call it('r',defs,'nonlinear','[R] NonLinear Optics Post-Processing')
 !
 !
 ! Temperatures
 !
 call it(defs,'ElecTemp','Electronic Temperature',Tel,T_unit,verb_level=V_general)
 call it(defs,'BoseTemp','Bosonic Temperature',Bose_Temp,T_unit,verb_level=V_general)
 !
 ! DOS
 !
 call it(defs,'DOS_broad', 'Broadening of the DOS',dos_broadening,E_unit) 
 call it(defs,'DOS_bands', 'DOS bands',dos_bands) 
 call it(defs,'DOSERange', 'Energy range',dos_E_range,unit=E_unit) 
 call it(defs,'DOSESteps', 'Energy steps',dos_E_steps) 
 call it('f',defs,'PrtOccOnly', 'Occupied states only') 
 !
 call it(defs,'WFReference', 'band and kpt index of reference WF for spin direction',WF_ref,verb_level=V_general)
 !
 !
 ! BANDS
 !
 call it(defs,'BANDS_steps', 'Number of divisions',BANDS_steps) 
 call it(defs,'INTERPGrid', 'Interpolation BZ Grid',interp_grid)
 call it(defs,'ShellFac','The bigger it is a higher number of shells is used',lpfac)
 !
 ! Wannier interface
 !
 call it(defs,'Seed','Seed name',seed_name)
 call it(defs,'WriteAMU','What to write (A, M, U)',what_to_write)
 !
 ! BZ grids
 !
 call it(defs,'cooIn','Points coordinates (in) cc/rlu/iku/alat',coo_in)
 call it(defs,'cooOut','Points coordinates (out) cc/rlu/iku/alat',coo_out)
 !
 !
 call it(defs,'BZ_random_Nk','BZ Random points to be printed',BZ_random_nkpts)
 !
 call it('f',defs,'NoWeights',    ' Do not print points weight') 
 call it('f',defs,'ForceUserPts', ' Do not check the correcteness of the user points') 
 call it('f',defs,'ListPts',      ' List the internal q/k points also in the parser format') 
 call it('f',defs,'ExpandPts',    ' Expand the internal q/k points in the BZ') 
 call it('f',defs,'DipWeight',    ' Weight the contribution to the exciton WFs with the dipoles') 
 !
 ! WFs
 !
 call it(defs,'FFTGvecs','[FFT] Plane-waves',wf_ng,G_unit)
 call it(defs,'Degen_Step', 'Maximum energy separation of two degenerate states',&
&                           deg_energy,E_unit)
#if defined _YPP_ELPH
 call it(defs,'ResTresh', '[o/o] Treshold on processed polaronic residuals',elph_residual_treshold)
#endif
 call it(defs,'MinWeight', 'Print transitions with weight => MinWeight',min_weight,verb_level=V_general)
 call it(defs,'States','Index of the BS state(s)',state_ctl)
 call it(defs,'Cells', 'Number of cell repetitions in each direction (odd or 1)',ncell)
 call it(defs,'Hole', '[cc] Hole position in unit cell',r_hole)
 call it(defs,'WFMult', 'Multiplication factor to the excitonic wavefunction',WF_multiplier,verb_level=V_general)
 !
 ! 
 ! PLOT
 !
 ! p_dir: plot cut in the a1,a2,a3 basis
 ! 
 ! p_format: (c)ube/(g)nuplot/(x)crysden
 !
 call it(defs,'Format', 'Output format [(c)ube/(g)nuplot/(x)crysden]',p_format)
 call it(defs,'Direction', '[rlu] [1/2/3] for 1d or [12/13/23] for 2d [123] for 3D',p_dir)
 call it(defs,'MagDir', 'Magnetization component',mag_dir)
 call it(defs,'CurrentDir', '[cc] Carthesian direction (used to average in 2D/3D plots or to plot in 1D)',current_dir)
 call it('f',defs,'PhaseTrick','[MAG] Phase trick for a better diagonalization')
#if defined _YPP_ELPH
 !
 !EL-PH
 !
 call it(defs,'DBsPATH',  'Path to the PW el-ph databases',elph_dbs_path)
 call it(defs,'PHfreqF',  'PWscf format file containing the phonon frequencies',ph_freqs_file)
 call it(defs,'PHmodeF',  'PWscf format file containing the phonon modes',ph_modes_file)
 call it(defs,'EfGbroad', 'Fermi surface broadening (Gamma)',elph_gamma_broad,E_unit)
 call it(defs,'PhBroad','Phonon broadening (Eliashberg & DOS)',ph_broad,E_unit)
 call it(defs,'EE_Fermi','External Fermi Level (Eliashberg)',elph_Ef,E_unit)
 call it(defs,'PhStps', 'Energy steps',elph_steps)
 call it('f',defs,'GkkpExpand','Expand the gkkp in the whole BZ')
 !
#endif
 !
 !BZ E Double Grid
 !
 call it(defs,'BZ_DbGd_Nk','BZ Number of points in the double grid',BZ_DbGd_nkpts_to_use)
 call it('f',defs,'SkipCheck' , 'Skip internal consistency check on k-point grids')
 call it('f',defs,'noBZExpand' , 'Keep the Double Grid k-points in the IBZ without using the symmetries')
 !
 !WFs perturbative SOC mapping
 !
 call it(defs,'SOC_DBs',"Path to the folder containing the DB with SOC",perturbative_SOC_path)
 call it(defs,'bands_to_map',"Number of bands to map",SOC_bands_to_map)
 call it(defs,'bands_buffer',"Bands to be used only as a buffer",SOC_bands_buffer)
 call it(defs,'SOC_splitting',"SOC splitting between the levels",SOC_split,E_unit) 
 !
 ! Wannier bands
 !
 call it(defs,'W90_fname','Name of w90 input file',w90_fname)
 call it(defs,'WannBnds','Range of interpolated bands, internally start from 1',wannier_bands)
 call it(defs,'DgridOrd','Order of interpolation with double grid',ord_dgrid_ipol)
 !
 !
 ! Remove Simmetries
 !
 if(n_spinor==2) call it('f',defs,'ExpandSymm' , 'Add inv and magnetic symms if possible')
 call it('f',defs,'RmAllSymm' , 'Remove all symmetries')
 call it('f',defs,'RmTimeRev' , 'Remove Time Reversal')
 call it(defs,'Efield1','First external Electric Field',E_field(1,:))
 call it(defs,'Efield2','Additional external Electric Field',E_field(2,:))
 !
 ! Non_linear response analysis
 !
 ! Post-Processing
 !
 call it(defs,'Xorder','Max order of the response functions',X_order)
 call it(defs,'EnRngeRt','Energy range',NL_response%W_range,unit=E_unit)
 call it(defs,'ETStpsRt','Total Energy steps',NL_response%W_n_steps)
 call it(defs,'TimeRange','Time-window where processing is done',NL_response%T_range,unit=Time_unit(1))
 call it(defs,'TimeStep','Time step',NL_response%delta_T,unit=Time_unit(1))
 call it(defs,'NRichardson','Order of Richardson extrapolation',N_richardson)
 call it(defs,'DampMode','Damping type ( NONE | LORENTZIAN | GAUSSIAN )',NL_response%damping)
 call it(defs,'DampFactor','Damping parameter',NL_response%damp_factor,unit=E_unit)
 !
end subroutine
